# Copyright 2018 ML2Grow BVBA
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from umongo.instance import LazyLoaderInstance
from .builder import DataStoreBuilder
from .helpers import DataStoreClientWrapper


class DataStoreInstance(LazyLoaderInstance):

    def __init__(self, *args, **kwargs):
        self.BUILDER_CLS = DataStoreBuilder  # pylint: disable=C0103
        super().__init__(*args, **kwargs)

    def init(self, db):
        super(DataStoreInstance, self).init(DataStoreClientWrapper(db))
